import os
import subprocess

def logout():
    whoami = (subprocess.check_output(['bash','-c','whoami']))[:-1]
    whoami = whoami.decode("utf-8")
    path = ("/home/"+whoami+"/.nethub_token.json")

    if os.path.isfile(path):
        os.remove(path)
        return True




import sys
import subprocess

def commit(msg=None):

    if msg == None:
        commit_msg = input('Commit: ')
        commit_command = 'git commit -m "%s"' % (commit_msg)
        subprocess.call(commit_command, shell=True)



import os
import sys
import h5py
import json
import numpy as np

def keras_to_netpy_merge(netpy_net, keras_net):

    with open(netpy_net+'/'+netpy_net+'_data/'+netpy_net+'.json', 'r') as master:
        master_data = json.load(master)
        num_of_master_layers = len(master_data['net']['arch']['layers'])
        num_of_master_connections = len(master_data['net']['arch']['connections'])
        num_of_master_in_neurons = master_data['net']['arch']['layers'][0]['num_of_neurons']
        num_of_master_modules = num_of_master_layers + num_of_master_connections

    with open(keras_net+'/'+keras_net+'_data/'+keras_net+'.json', 'r') as second:
        second_data = json.load(second)
        num_of_second_layers = len(second_data["config"])
        num_of_second_in_neurons = second_data["config"][0]["config"]["batch_input_shape"][1]
        name_of_second_in_layer = second_data["config"][0]["config"]["name"]
        name_of_second_hid_layer = second_data["config"][1]["config"]["name"]
    
    if num_of_master_layers != num_of_second_layers+1:
        print('Non-valid architecture')
        return False
    elif num_of_master_in_neurons != num_of_second_in_neurons:
        print('Non-valid architecture')
    else:
        netpy_weights = h5py.File(netpy_net+'/'+netpy_net+'_data/'+netpy_net+'.h5', 'r+')
        keras_weights = h5py.File(keras_net+'/'+keras_net+'_data/'+keras_net+'.h5', 'r')

        current_master_weights = netpy_weights['weights_0_2'][:]
        current_second_weights = keras_weights[name_of_second_in_layer][name_of_second_in_layer]["kernel:0"][:]

        cur_master_line = len(current_master_weights)
        cur_master_col = len(current_master_weights[0])
        cur_second_line = len(current_second_weights)
        cur_second_col = len(current_second_weights[0])

        current_master_weights = np.append(current_master_weights, current_second_weights, axis=1)

        netpy_weights['weights_0_2'].resize((cur_master_line, cur_master_col+cur_second_col))
        netpy_weights['weights_0_2'][:] = current_master_weights

        current_master_weights = netpy_weights['weights_2_4'][:]
        current_second_weights = keras_weights[name_of_second_hid_layer][name_of_second_hid_layer]["kernel:0"]

        cur_master_line = len(current_master_weights)
        cur_master_col = len(current_master_weights[0])
        cur_second_line = len(current_second_weights)
        cur_second_col = len(current_second_weights[0])

        right_top_matrix = np.zeros((cur_master_line, cur_second_col))
        left_bot_matrix = np.zeros((cur_second_line, cur_master_col))

        current_master_weights = np.append(current_master_weights, right_top_matrix, axis=1)
        bottom_matrix = np.append(left_bot_matrix, current_second_weights, axis=1)

        current_master_weights = np.vstack((current_master_weights, bottom_matrix))

        netpy_weights['weights_2_4'].resize((cur_master_line+cur_second_line,
                                             cur_master_col+cur_second_col))

        netpy_weights['weights_2_4'][:] = current_master_weights

    netpy_weights.close()
    keras_weights.close()

    with open(netpy_net+'/'+netpy_net+'_data/'+netpy_net+'.json', 'r') as log:
        json_data = json.load(log)
        json_data['net']['arch']['layers'][-2]['num_of_neurons'] = len(current_master_weights)
        json_data['net']['arch']['layers'][-1]['num_of_neurons'] = len(current_master_weights[0])

    with open(netpy_net+'/'+netpy_net+'_data/'+netpy_net+'.json', 'w') as log:
        log.write(json.dumps(json_data, indent=4))



import sys
import json
import requests
import subprocess

from .login import login
from .graphql import create_project_mutation

from .constants import (
    whoami,
    config_path,
)

def server_init(title=None):
    if title==None:
        title = sys.argv[2]


    with open(config_path, "r") as config_file:
        config_data = json.load(config_file)
        token = config_data["token"]

    if token == None:
        login()

    conf_net_path = title+"/conf.json"

    with open(config_path, "r") as config_file:
        config_data = json.load(config_file)
        token = config_data["token"]
        server_adress = config_data["serverAdress"]

    token = 'jwt {}'.format(token)

    with open(conf_net_path, "r") as net_conf:
        json_data = json.load(net_conf)
        name = json_data["name"]
        lib_name = json_data["library"]

    mutation = create_project_mutation(name, lib_name)

    response = requests.post(
        server_adress+'graphql/',
        json={
            'query': mutation
        },
        headers={
            'Authorization': token
        },
    )

    code = int(response.status_code)

    if code == 200:
        response_json = response.json()
        git_adress = response_json['data']['createProject']['project']['gitAdress']

        remote_add_origin_command = 'cd %s && git remote add origin %s'
        commit_command = 'cd %s && git add . && git commit -m "%s"'
        push_command = 'cd %s && git push -u origin master'

        commit_msg = input('Commit: ')

        subprocess.call(remote_add_origin_command % (title, git_adress), shell=True)
        subprocess.call(commit_command % (title, commit_msg), shell=True)
        subprocess.call(push_command % (title), shell=True)

        with open(title+"/conf.json", 'r') as conf:
            json_data = json.load(conf)
            json_data["link"] = git_adress

        with open(title+"/conf.json", 'w') as conf:
            json.dump(json_data, conf, indent=4)

        return True
    else:
        print(response.content)
        return False



import requests
import json
import getpass
import subprocess
import os.path

from .constants import (
    whoami,
    config_path,
)

from .graphql import token_auth_mutation

def login():

    with open(config_path, 'r') as config_file:
        config_data = json.load(config_file)
        server_adress = config_data["serverAdress"]

    username = str(input('Login: '))
    password = getpass.getpass('Password:')

    mutation = token_auth_mutation(username, password)

    response = requests.post(
        server_adress+'graphql/',
        json={
            'query': mutation
        },
    )

    if int(response.status_code) == 200:
        response_json = response.json()
        token = str(response_json['data']['tokenAuth']['token'])

        with open(config_path, 'r') as config_file:
            config_data = json.load(config_file)
            config_data['token'] = token

        with open(config_path, 'w') as config_file:
            config_file.write(json.dumps(config_data, indent=4))

        print('Login successful!')
        return True

    """
    Для тестирования, что бы не вводить логин и пароль

        # response = requests.post("http://127.0.0.1:21031/api-token-auth/",
        # data={'username': 'root', 'password': 'vaz2107top' })

        # Получаем токен
        token = response.json()
        # Получаю код состояния http
        code = int(response.status_code)
        # Если логин и пароль верны, то сохранить токен
        if code == 200:
            with open(path, 'w') as outfile:
                json.dump(token, outfile)
            print("Login successful")
            return True
        else:
            # Если логи и пароль не верны print ошибки
            str_token = json.dumps(token)
            a,b = str_token.split('"]')
            c,d = a.split('["')
            print("Error code:", code)
            print(d)
            return False
        """



from manager.core.managment.commands.merges.keras_to_keras import keras_to_keras_merge
from manager.core.managment.commands.merges.netpy_to_netpy import netpy_to_netpy_merge
from manager.core.managment.commands.merges.keras_to_netpy import keras_to_netpy_merge
from manager.core.managment.commands.merges.netpy_to_keras import netpy_to_keras_merge



import sys
import os

from .commands.hello import hello
from .commands.newnet import newnet
from .commands.fork import fork
from .commands.merge import merge
from .commands.login import login
from .commands.logout import logout
from .commands.server_init import server_init
from .commands.pull import pull
from .commands.push import push
from .commands.add import add
from .commands.commit import commit
from .commands.init import init

from .commands.constants import config_path

class ManageUtil:
    def __init__(self, argv=None):
        self.argv = argv or sys.argv[:]

    def execute(self):
        subcommand = self.argv[1]

        #if not os.path.isfile(config_path):
        #    init()

        if subcommand == 'login':
            login()

        if subcommand == 'logout':
            logout()

        if subcommand == 'hello':
            hello()

        if subcommand == 'newnet':
            newnet()

        if subcommand == 'fork':
            fork()

        if subcommand == 'merge':
            merge()

        if subcommand == 'serverinit':
            server_init()

        if subcommand == 'push':
            push()

        if subcommand == 'pull':
            pull()

        if subcommand == 'add':
            add()

        if subcommand == 'commit':
            commit()

        if subcommand == 'init':
            init()

def execute_from_command_line(argv=None):
    util = ManageUtil(argv)
    util.execute()



mutation = '''
    mutation {
        tokenAuth(username: "%s", password: "%s") {
            token
        }
    }
'''

def token_auth_mutation(username, password):
    return mutation % (username, password)



import os
import sys
import h5py
import json
import numpy as np

def save_attributes_to_hdf5_group(group, name, data):

    HDF5_OBJECT_HEADER_LIMIT = 64512

    bad_attributes = [x for x in data if len(x) > HDF5_OBJECT_HEADER_LIMIT]

    # Expecting this to never be true.
    if len(bad_attributes) > 0:
        raise RuntimeError('The following attributes cannot be saved to HDF5 '
                           'file because they are larger than %d bytes: %s'
                           % (HDF5_OBJECT_HEADER_LIMIT,
                              ', '.join([x for x in bad_attributes])))

    data_npy = np.asarray(data)

    num_chunks = 1
    chunked_data = np.array_split(data_npy, num_chunks)

    # This will never loop forever thanks to the test above.
    while any(map(lambda x: x.nbytes > HDF5_OBJECT_HEADER_LIMIT, chunked_data)):
        num_chunks += 1
        chunked_data = np.array_split(data_npy, num_chunks)

    if num_chunks > 1:
        for chunk_id, chunk_data in enumerate(chunked_data):
            group.attrs['%s%d' % (name, chunk_id)] = chunk_data
    else:
        group.attrs[name] = data


def keras_to_keras_merge(master_net, second_net):
    print("Coming soon!")

def keras_to_keras_merge_test(master_net, second_net):
    
    with open(master_net+'/'+master_net+'_data/'+master_net+'.json', 'r') as master:
        master_data = json.load(master)
        num_of_master_layers = len(master_data["config"])
        num_of_master_in_neurons = master_data["config"][0]["config"]["batch_input_shape"][1]
        name_of_master_in_layer = master_data["config"][0]["config"]["name"]
        name_of_master_hid_layer = master_data["config"][1]["config"]["name"]

    with open(second_net+'/'+second_net+'_data/'+second_net+'.json', 'r') as second:
        second_data = json.load(second)
        num_of_second_layers = len(second_data["config"])
        num_of_second_in_neurons = second_data["config"][0]["config"]["batch_input_shape"][1]
        name_of_second_in_layer = second_data["config"][0]["config"]["name"]
        name_of_second_hid_layer = second_data["config"][1]["config"]["name"]


    if num_of_master_layers != num_of_second_layers:
        print('Non-valid architecture')
        return False
    elif num_of_master_in_neurons != num_of_second_in_neurons:
        print('Non-valid architecture')
    else:
        master_weights = h5py.File(master_net+'/'+master_net+'_data/'+master_net+'.h5', 'r')
        second_weights = h5py.File(second_net+'/'+second_net+'_data/'+second_net+'.h5', 'r')


        current_master_weights = master_weights[name_of_master_in_layer][name_of_master_in_layer]["kernel:0"][:]
        current_second_weights = second_weights[name_of_second_in_layer][name_of_second_in_layer]["kernel:0"][:]

        cur_master_line = len(current_master_weights)
        cur_master_col = len(current_master_weights[0])
        cur_second_line = len(current_second_weights)
        cur_second_col = len(current_second_weights[0])

        print(len(current_master_weights), len(current_master_weights[0]), "/n")

        current_master_weights = np.append(current_master_weights, current_second_weights, axis=1)

        print(len(current_master_weights), len(current_master_weights[0]), "/n")

        current_master_weights = np.array(current_master_weights, dtype='float32')


        data_new = h5py.File(master_net+"/"+master_net+"_data/tmp.h5", "w")

        backend = "tensorflow"
        version = "2.1.4"
        layers = [name_of_master_in_layer, name_of_master_hid_layer]

        data_new.attrs["backend"] = str(backend).encode('utf8')
        data_new.attrs["keras_version"] = str(version).encode('utf8')
        save_attributes_to_hdf5_group(data_new, 
                                      "layer_names", 
                                      [layer.encode('utf8') for layer in layers],
                                     )

        group = data_new.create_group(name_of_master_in_layer)
       
        weight_names = [name_of_master_in_layer+"/kernel:0",name_of_master_in_layer+"/bias:0"]

        save_attributes_to_hdf5_group(group, 
                                      "weight_names", 
                                      [weight.encode('utf8') for weight in weight_names],
                                     )

        group2 = data_new.create_group(name_of_master_in_layer + "/" +name_of_master_in_layer)

        param_dset = group2.create_dataset("kernel:0",
                                           data=current_master_weights,
                                           maxshape=(None, None)
                                          )

        param_dset[:] = np.asarray(current_master_weights)

        group2.create_dataset("bias:0",
                              data=[0],
                             )
        
        current_master_weights = master_weights[name_of_master_hid_layer][name_of_master_hid_layer]["kernel:0"]
        current_second_weights = second_weights[name_of_second_hid_layer][name_of_second_hid_layer]["kernel:0"]

        cur_master_line = len(current_master_weights)
        cur_master_col = len(current_master_weights[0])
        cur_second_line = len(current_second_weights)
        cur_second_col = len(current_second_weights[0])

        right_top_matrix = np.zeros((cur_master_line, cur_second_col))
        left_bot_matrix = np.zeros((cur_second_line, cur_master_col))

        current_master_weights = np.append(current_master_weights, right_top_matrix, axis = 1)

        bottom_matrix = np.append(left_bot_matrix, current_second_weights, axis=1)

        current_master_weights = np.vstack((current_master_weights, bottom_matrix))
        current_master_weights = np.array(current_master_weights, dtype='float32')
        

        group = data_new.create_group(name_of_master_hid_layer)

        weight_names = [
                        name_of_master_hid_layer+"/kernel:0",
                        name_of_master_hid_layer+"/bias:0",
                       ]

        save_attributes_to_hdf5_group(group, 
                                      "weight_names", 
                                      [weight.encode('utf8') for weight in weight_names],
                                     )


        group2 = data_new.create_group(name_of_second_hid_layer + "/" + name_of_master_hid_layer)

        param_dset = group2.create_dataset("kernel:0", 
                                           data=current_master_weights,
                                           maxshape=(None, None),
                                          )

        param_dset[:] = np.asarray(current_master_weights)

        group2.create_dataset("bias:0",
                              data=[0],
                             )

        data_new.flush()

        master_weights.close()
        second_weights.close()

        os.remove(master_net+"/"+master_net+"_data"+"/"+master_net+".h5")
        os.rename(master_net+"/"+master_net+"_data/tmp.h5", 
                  master_net+"/"+master_net+"_data"+"/"+master_net+".h5",
                 )

        with open(master_net+'/'+master_net+'_data/'+master_net+'.json', 'r') as config:
            json_data = json.load(config)
            json_data["config"][0]["config"]["units"] += cur_second_line
            json_data["config"][1]["config"]["units"] += cur_second_col

        with open(master_net+'/'+master_net+'_data/tmp.json', 'w') as config:
            config.write(json.dumps(json_data, indent=4))

        os.remove(master_net+"/"+master_net+"_data/"+master_net+".json")
        os.rename(master_net+"/"+master_net+"_data/tmp.json",
                  master_net+"/"+master_net+"_data/"+master_net+".json"
                 )



import os
import sys
import json
import importlib


def merge():
    master_net = sys.argv[2]
    second_net = sys.argv[3]

    if not os.path.isdir(master_net):
        print("Can't find net with name %s" % (master_net))
    elif not os.path.isdir(second_net):
        print("Can't find net with name %s" % (second_net))
    else:

        with open(master_net+'/conf.json', 'r') as master_conf:
            master_data = json.load(master_conf)
            master_lib = master_data["library"]

        with open(second_net+'/conf.json', 'r') as second_conf:
            second_data = json.load(second_conf)
            second_lib = second_data["library"]

        print("Merge " + second_lib + " to " + master_lib)

        function = second_lib + '_to_' + master_lib + '_merge'
        module = importlib.import_module("manager.core.managment.commands.merges")
        merge = getattr(module , function)


        merge(master_net, second_net)

        print('Successfully merge %s -> %s' % (second_net, master_net))




from setuptools import setup
from setuptools import find_packages


setup(name='nethub-manager',
      version='0.0.4',
      author='Nethub',
      author_email='nethub@yandex.ru',
      scripts=['manager/bin/nethub-manager.py'],
      entry_points={'console_scripts': [
          'nhm = manager.core.managment: execute_from_command_line'
      ]},
      install_requires=['h5py'],
      include_package_data=True,
      packages=find_packages()
)



import os
import sys
import json
import h5py
import shutil
import fileinput
import subprocess

from manager import templates
from .server_init import server_init

# Function for copy
def copy_files(name, lib_name, file_path):
    lib_name += "_template"
    template_path = templates.__path__[0] + '/' + lib_name
    shutil.copy2(template_path + name, file_path)

# Function for create empty files
def create_empty_file(file_path):
    with open(file_path, 'a'):
        os.utime(file_path, None)

# Function for replace str line
def replace_str(file_path, search_exp, replace_exp):
    for line in fileinput.input(file_path, inplace=1):
        if search_exp in line:
            line = line.replace(search_exp, replace_exp)
        sys.stdout.write(line)

# Main func
def newnet(name=None):

    if name == None:
        name = sys.argv[2]

    libs = {"netpy", "keras"}

    # print("1.NetPy\n2.Keras")
    # lib_name = input("Print your choice: ").lower()

    lib_name = 'netpy'

    if lib_name not in libs:
        print("Unknown library!")
        return False

    # Directory where project is
    top_dir = str(name)

    # Directory with metadata for project
    data_dir = top_dir+'/net_data'

    # Directory with dataset
    dataset_dir = top_dir+'/data_set'

    # File with network settings
    init_file = top_dir + '/network.py'

    # File with network settings for testing
    test_file = top_dir + '/test.py'

    # File with network settings for training
    train_file = top_dir + '/train.py'

    # File with network settings for iq testing
    iq_test_file = top_dir + '/iq_test.py'


    # Json file with project settings
    settings_file = top_dir + '/conf.json'

    # Train data set
    train_Y, train_X = dataset_dir + '/train_Y.txt', dataset_dir + '/train_X.txt'

    # Test data set
    test_Y, test_X = dataset_dir + '/test_Y.txt', dataset_dir + '/test_X.txt'


    # If path does not exist
    if not os.path.isdir(top_dir):
        os.makedirs(top_dir)
        os.makedirs(data_dir)
        os.makedirs(dataset_dir)

        create_empty_file(train_Y)
        create_empty_file(train_X)

        create_empty_file(test_Y)
        create_empty_file(test_X)

        subprocess.call("cd "+top_dir+" && git init", shell=True)

        copy_files("/network.pyt", lib_name, init_file)
        copy_files("/test.pyt", lib_name, test_file)
        copy_files("/train.pyt", lib_name, train_file)
        copy_files("/iq_test.pyt", lib_name, iq_test_file)

        settings = {
                    'name': name,
                    'data_dir': 'net_data/',
                    'library': lib_name,
                    'link': None,
                   }

        with open(settings_file, 'w') as file:
            json.dump(settings, file, indent=4)

        if lib_name == "netpy":
            meta_data = {
                     'net': {
                        'name': name,
                        'total_epoch': 0,
                        'parent': None,
                        'loss': None,

                        'arch': {
                            'layers': [],
                            'connections': []
                        },

                        'iq_test': {
                            'iq': None,
                            'iq_min': None,
                            'iq_max': None,
                            'num_of_tests': None,
                        }
                     },

                     'train_parameters': {
                        'teacher': None,
                        'learning_rate': None,
                        'error': None,
                     },

                     'description': ''
                    }

            with open(data_dir + '/arch.json', 'w') as file:
                json.dump(meta_data, file, indent=4)


                old_line = "{name}"
                new_line = "name = '" +name+ "'"

                files = [init_file, test_file, train_file, iq_test_file]

                for i in range(len(files)):
                    replace_str(files[i], old_line, new_line)

        if lib_name == "keras":
            json_path_tmp = "{json_path}"
            weights_path_tmp = "{weights_path}"

            json_path = '"' + name + "_data" + '/' + name + '.json' + '"'
            weights_path = '"' + name + "_data" + '/' + name + '.h5' + '"'

            files = [init_file, train_file, test_file]

            for i in range(len(files)):
                replace_str(files[i], json_path_tmp, json_path)
                replace_str(files[i], weights_path_tmp, weights_path)


        server_init(name)
    else:
        print("I can't create net with this name :(")



import os
import sys
import h5py
import json
import numpy as np

def netpy_to_netpy_merge(master_net, second_net):

    with open(master_net+'/net_data/arch.json', 'r') as master:
        master_data = json.load(master)
        num_of_master_layers = len(master_data['net']['arch']['layers'])
        num_of_master_connections = len(master_data['net']['arch']['connections'])
        num_of_master_in_neurons = master_data['net']['arch']['layers'][0]['num_of_neurons']
        num_of_master_modules = num_of_master_layers + num_of_master_connections

    with open(second_net+'/net_data/arch.json', 'r') as second:
        second_data = json.load(second)
        num_of_second_layers = len(second_data['net']['arch']['layers'])
        num_of_second_connections = len(second_data['net']['arch']['connections'])
        num_of_second_in_neurons = second_data['net']['arch']['layers'][0]['num_of_neurons']
        num_of_second_modules = num_of_second_layers + num_of_second_connections

    if num_of_master_layers != num_of_second_layers:
        print('Non-valid architecture.')
    elif num_of_master_in_neurons!= num_of_second_in_neurons:
        print('Non-valid architecture.')
    else:

        master_weights = h5py.File(master_net+'/net_data/weights.h5', 'r+')
        second_weights = h5py.File(second_net+'/net_data/weights.h5', 'r+')


        current_master_weights = master_weights['weights_0_2'][:]
        current_second_weights = second_weights['weights_0_2'][:]

        cur_master_line = len(current_master_weights)
        cur_master_col = len(current_master_weights[0])
        cur_second_line = len(current_second_weights)
        cur_second_col = len(current_second_weights[0])

        current_master_weights = np.append(current_master_weights, current_second_weights, axis=1)

        master_weights['weights_0_2'].resize((cur_master_line, cur_master_col+cur_second_col))
        master_weights['weights_0_2'][:] = current_master_weights

        for i in range(3, num_of_master_modules, 2):
            current_master_weights = master_weights['weights_'+str(i-1)+'_'+str(i+1)][:]
            current_second_weights = second_weights['weights_'+str(i-1)+'_'+str(i+1)][:]

            cur_master_line = len(current_master_weights)
            cur_master_col = len(current_master_weights[0])
            cur_second_line = len(current_second_weights)
            cur_second_col = len(current_second_weights[0])

            right_top_matrix = np.zeros((cur_master_line, cur_second_col))
            left_bot_matrix = np.zeros((cur_second_line, cur_master_col))

            current_master_weights = np.append(current_master_weights, right_top_matrix, axis=1)
            bottom_matrix = np.append(left_bot_matrix, current_second_weights, axis=1)

            current_master_weights = np.vstack((current_master_weights, bottom_matrix))

            master_weights['weights_'+str(i-1)+'_'+str(i+1)].resize((cur_master_line+cur_second_line,
                                                                     cur_master_col+cur_second_col))
            master_weights['weights_'+str(i-1)+'_'+str(i+1)][:] = current_master_weights

        master_weights.close()

        with open(master_net+'/net_data/arch.json', 'r') as log:
            json_data = json.load(log)
            json_data['net']['arch']['layers'][-2]['num_of_neurons'] = len(current_master_weights)
            json_data['net']['arch']['layers'][-1]['num_of_neurons'] = len(current_master_weights[0])

        with open(master_net+'/net_data/arch.json', 'w') as log:
            log.write(json.dumps(json_data, indent=4))





mutation = '''
    mutation {
        createProject(input: {
            title:"%s",
            library:"%s"
        }) {
            project {
                gitAdress
            }
        }
    }
'''

def create_project_mutation(name, lib_name):
    return mutation % (name, lib_name)



import os
import sys
import json
import shutil

from .newnet import newnet, replace_str
from distutils.dir_util import copy_tree


def copy_file(first_net, second_net ,file_name):
    shutil.copy2(first_net+'/'+file_name, second_net+'/'+file_name)

def path_to_h5_file(netname):
    return netname+'/'+netname+'_data/'+netname+'.h5'

def fork():
    first_net = sys.argv[2]
    second_net = sys.argv[3]

    if not os.path.isdir(first_net):
        print("Can't find net with this name :(")
    elif os.path.isdir(second_net):
        print("Net with name %s already exist!" % (second_net))
    else:

        if not os.path.isdir(second_net):
            newnet(str(second_net))

            copy_file(first_net, second_net, 'network.py')
            copy_file(first_net, second_net, 'test.py')
            copy_file(first_net, second_net, 'train.py')
            copy_file(first_net, second_net, 'iq_test.py')

            files = [
                '/network.py',
                '/test.py',
                '/train.py',
                '/iq_test.py',
            ]

            old_line = "net = FeedForwardNet(name = '" +first_net+"')"
            new_line = "net = FeedForwardNet(name = '" +second_net+"')"

            for i in range(len(files)):
                replace_str(second_net+files[i], old_line, new_line)

            copy_tree(first_net+'/data_set', second_net+'/data_set')

            if os.path.isfile(path_to_h5_file(first_net)):
                shutil.copy2(
                    path_to_h5_file(first_net),
                    path_to_h5_file(second_net)
                )

            shutil.copy2(
                first_net+'/'+first_net+'_data/'+first_net+'.json',
                second_net+'/'+second_net+'_data/parent_data.json'
            )

            shutil.copy2(
                first_net+'/'+first_net+'_data/'+first_net+'.json',
                second_net+'/'+second_net+'_data/'+second_net+'.json'
            )


            with open(second_net+'/'+second_net+'_data/'+second_net+'.json', 'r') as log:
                json_data = json.load(log)
                json_data['net']['name'] = second_net
                json_data['net']['total_epoch'] = 0
                json_data['net']['parent'] = first_net
                json_data['net']['loss'] = 0
                json_data['net']['iq_test']['iq'] = 0
                json_data['net']['iq_test']['iq_min'] = 0
                json_data['net']['iq_test']['iq_max'] = 0
                json_data['net']['iq_test']['num_of_tests'] = 0

                json_data['train_parameters']['teacher'] = None
                json_data['train_parameters']['learning_rate'] = None
                json_data['train_parameters']['error'] = None

            with open(second_net+'/'+second_net+'_data/'+second_net+'.json', 'w') as log:
                log.write(json.dumps(json_data, indent=4))



import json
import subprocess

from .login import login
from .constants import (
    whoami,
    config_path,
)

def init():
    config = {
        'serverAdress': None,
        'gitServerAdress': None,
        'token': None,
        'nets': []
    }

    server_adress = input("Server adress (default: 'http://127.0.0.1:21020/'): ")

    if server_adress == '':
        server_adress = 'http://127.0.0.1:21020/'

    git_server_adress = server_adress[:-2]+'6/'

    config['serverAdress'] = server_adress
    config['gitServerAdress'] = git_server_adress

    with open(config_path, 'w') as config_file:
        json.dump(config, config_file, indent=4)

    # login()
